--- Initialise the database. Saved variables are restored by the WoW loader after all this code is run, so it's safe just to initialise the variables here and allow them to be overwritten.
tally_npcs = {}
tally_npcs_per_character = {}

--- Finds the existing or creates a new NPC database entry for npcId. Internal to TallyDatabase.
-- @param npcId Integer. The ID of the type of NPC to find or create.
function Tally.find_npc(npcId)
	local entry = tally_npcs[npcId]
	if entry == nil then
		entry =
		{
			name = nil,
			loots = 0,
			locations = {},
			loot = {}
		}
		tally_npcs[npcId] = entry
	end
	return entry
end

--- Finds the existing or creates a new per-character NPC database entry for npcId. Internal to TallyDatabase.
-- @param npcId Integer. The ID of the type of NPC to find or create.
function Tally.find_npc_per_character(npcId)
	local entry = tally_npcs_per_character[npcId]
	if entry == nil then
		entry =
		{
			kills = 0,
			loots = 0,
		}
		tally_npcs_per_character[npcId] = entry
	end
	return entry
end

--- Notes details of a mob that has presumably just been observed.
-- @param npcId Integer. The ID of the type of NPC that has been looted.
-- @param name String. The mob's (localised) name. This will be the display name for mobs with this ID in future.
-- @param location String. The zone name and subzone name where the mob should be noted as spawning.
function Tally.record_mob(npcId, name, location)
	local entry = Tally.find_npc(npcId)
	entry.name = name
	local locations = entry.locations
	if locations[location] == nil then
		locations[location] = 1
	end
end

--- Notes that a mob of type npcId has just been killed.
-- @param npcId Integer. The ID of the type of NPC that has been looted.
function Tally.record_mob_kill(npcId)
	local entry_per_character = Tally.find_npc_per_character(npcId)
	entry_per_character.kills = entry_per_character.kills + 1
end

--- Notes that a mob of type npcId has just been looted and has dropped items.
-- @param npcId Integer. The ID of the type of NPC that has been looted.
function Tally.record_mob_loot(npcId)
	local entry = Tally.find_npc(npcId)
	entry.loots = entry.loots + 1
	local entry_per_character = Tally.find_npc_per_character(npcId)
	entry_per_character.loots = entry_per_character.loots + 1
end

--- Notes that an item of type itemId has just been looted from one or more corpses.
-- @param itemId Integer. The ID of the type of item that has dropped.
-- @param items Table. A table of npcId -> quantity.
function Tally.record_item_loot_by_mob(itemId, mobs)
	for npcId, quantity in pairs(mobs) do
		local entry = Tally.find_npc(npcId)
		entry.loots = entry.loots + 1
		local loot = entry.loot
		if loot[itemId] == nil then
			loot[itemId] = quantity
		else
			loot[itemId] = loot[itemId] + quantity
		end
		local entry_per_character = Tally.find_npc_per_character(npcId)
		entry_per_character.loots = entry_per_character.loots + 1
	end
end

--- Notes that a mob of type npcId has just been observed running away in fear.
-- @param npcId Integer. The ID of the type of NPC that runs.
function Tally.record_mob_runs(npcId)
	local entry = Tally.find_npc(npcId)
	entry.runs = 1
end

function Tally.record_money_by_mob(t)
-- To do - code this
end
